import math

import matplotlib.pyplot as plt
import os
import gdal
import numpy as np

def readTif(fileName):
    dataset = gdal.Open(fileName)
    if dataset == None:
        print(fileName + "The file could not be opened")
    return dataset


# Save the TIF file function
def writeTiff(im_data, im_geotrans, im_proj, path):
    if 'int8' in im_data.dtype.name:
        datatype = gdal.GDT_Byte
    elif 'int16' in im_data.dtype.name:
        datatype = gdal.GDT_UInt16
    else:
        datatype = gdal.GDT_Float32
    if len(im_data.shape) == 3:
        im_bands, im_height, im_width = im_data.shape
    elif len(im_data.shape) == 2:
        im_data = np.array([im_data])
        im_bands, im_height, im_width = im_data.shape
    # Create a file
    driver = gdal.GetDriverByName("GTiff")
    dataset = driver.Create(path, int(im_width), int(im_height), int(im_bands), datatype)
    print(dataset)
    if (dataset != None):
        dataset.SetGeoTransform(im_geotrans)  # Writes an affine transform parameter
        dataset.SetProjection(im_proj)  # Write the projection
    for i in range(im_bands):
        dataset.GetRasterBand(i + 1).WriteArray(im_data[i])
    del dataset

def calTVDI(NDVIPath, LSTPath, NDVI_range, outfig, SavePath):
    # Read input grid data information
    print("READ input raster data information...")
    dataset = readTif(NDVIPath)
    width = dataset.RasterXSize
    height = dataset.RasterYSize
    NDVI_geotrans = dataset.GetGeoTransform()
    NDVI_proj = dataset.GetProjection()
    NDVI = dataset.ReadAsArray(0, 0, width, height)
    dataset = readTif(LSTPath)
    LST = dataset.ReadAsArray(0, 0, width, height)
    LST = LST*0.02 - 273

    abcd = edge(NDVI, LST, NDVI_range, outfig)
    a = abcd[0][0]
    b = abcd[0][1]
    c = abcd[1][0]
    d = abcd[1][1]
    print("Dry Edge fitting equation：y = %.3fx + %.3f，r2 = %.3f" % (a, b, abcd[0][2] * abcd[0][2]))
    print("Wet edge fitting equation：y = %.3fx + %.3f，r2 = %.3f" % (c, d, abcd[1][2] * abcd[1][2]))
    print("Compute TVDI...")
    TVDI = np.zeros(NDVI.shape)
    for m in range(NDVI.shape[0]):
        for n in range(NDVI.shape[1]):
            TVDI[m][n] = (LST[m][n] - (abcd[1][0] * NDVI[m][n] + abcd[1][1])) / (
                        (abcd[0][0] * NDVI[m][n] + abcd[0][1]) - (abcd[1][0] * NDVI[m][n] + abcd[1][1]))
    TVDI[TVDI < 0] = 0
    TVDI[TVDI > 1] = 1
    #  Save TVDI
    writeTiff(TVDI, NDVI_geotrans, NDVI_proj, SavePath)
    print("The calculation is complete and saved as '%s'\n" % SavePath)


def edge(NDVI, LST, NDVI_range, outfig):
    print("Dry Edge and wet edge fit...")
    edgeArr = [[] for _ in range(99)]
    for i in range(NDVI.shape[0]):
        for j in range(NDVI.shape[1]):
            if NDVI[i][j] > NDVI_range[0] - 0.01 and NDVI[i][j] < NDVI_range[1] + 0.01 and LST[i][j]>-20 and LST[i][j]<50:
                k = '%.2f' % NDVI[i][j]
                edgeArr[int(float(k) * 100)].append(LST[i][j])

    # Sort and look for dry and wet edges
    eMax = []
    NDVISort = []
    eMin = []
    for n in range(len(edgeArr)):
        edgeSort = np.sort(edgeArr[n])
        if len(edgeSort) > 1:
            eMax.append(edgeSort[-1])
            eMin.append(edgeSort[0])
            NDVISort.append(float(n) / 100.)

    fitMax = linefit(NDVISort, eMax)
    fitMin = linefit(NDVISort, eMin)

    # 作图
    createPlot(NDVISort, eMax, eMin, [fitMax, fitMin], outfig)

    return [fitMax, fitMin]


#  Fitting equations
def linefit(x, y):
    N = float(len(x))
    sx, sy, sxx, syy, sxy = 0, 0, 0, 0, 0
    for i in range(0, int(N)):
        sx += x[i]
        sy += y[i]
        sxx += x[i] * x[i]
        syy += y[i] * y[i]
        sxy += x[i] * y[i]
    a = (sy * sx / N - sxy) / (sx * sx / N - sxx)
    b = (sy - a * sx) / N
    r = abs(sy * sx / N - sxy) / math.sqrt((sxx - sx * sx / N) * (syy - sy * sy / N))
    return [a, b, r]


#  Draw dry edge wet edge fitting diagram
def createPlot(NDVI, emax, emin, abcd, outfig):
    plt.rcParams['font.family'] = 'Times New Roman'
    fig, ax = plt.subplots(figsize=(8, 5))
    plt.tick_params(labelsize=16)

    #  Set the upper and lower bounds of the y and x axes
    ylimit_min = np.min(emin) - 0.3 * np.average(emin)
    ylimit_max = np.max(emax) + 0.3 * np.average(emax)
    plt.ylim(ylimit_min, ylimit_max)
    plt.xlim(0.5, 1)

    #  Draw a fitting equation for the wet and dry edges
    cof = np.polyfit(NDVI, emax, 1)
    p = np.poly1d(cof)
    plt.plot(NDVI, p(NDVI), label='Dry edge fitted line', c='g')
    cof = np.polyfit(NDVI, emin, 1)
    p = np.poly1d(cof)
    plt.plot(NDVI, p(NDVI), label='Wet edge fitted line', c='y')

     # Draw the dry and wet edges
    plt.scatter(NDVI, emax, label='Dry edge', c='r')
    plt.scatter(NDVI, emin, label='Wet edge', c='b')
    #
    # plt.scatter(NDVI, emax, c='k')
    # plt.scatter(NDVI, emin, c='k')
    #  Set Y-axis and x-axis annotations
    ax.set_xlabel('NDVI', fontsize=16)
    ax.set_ylabel('LST ($^o$C)', fontsize=16)
    plt.legend(fontsize=16, loc='upper right', ncol=2)
    fig.tight_layout()
    plt.savefig(outfig, dpi=300)
    # plt.show()
    # plt.draw()
    # plt.pause(0.1)# Number of seconds in the interval: 6s
    fig1 = plt.figure(1)
    plt.close(fig1)

## Effective NDVI
rangeA = [0.6, 0.95]

# NDVIFolder = r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\NDVI-SGFilter\12-3month 0.1-0.65"
NDVIFolder = r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\NDVI-SGFilter"
LSTFolder = r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\08LST-Correct-new"

NDVINameList = os.listdir(NDVIFolder)
LSTNameList = os.listdir(LSTFolder)

for i in range(len(NDVINameList)):
    print(NDVINameList[i][0:7])
    print("==========================")
    NDVIPath = NDVIFolder + "/" + NDVINameList[i]
    # EVIPath = NDVIFolder + "/" + EVINameList[i]
    LSTPath = LSTFolder + "/" + LSTNameList[i]

    outfigNDVI = r"C:\Users\Administrator\Desktop\data\modis\TVDI\NDVI-fittedline/" + NDVINameList[i][0:7] + ".jpg"

    SavePathNDVI = r"C:\Users\Administrator\Desktop\data\modis\TVDI\NDVI-result-new/" + NDVINameList[i][0:7] + "TVDI.tif"

    calTVDI(NDVIPath, LSTPath, rangeA, outfigNDVI, SavePathNDVI)
